import { app, BrowserWindow, globalShortcut, ipcMain, Menu, Tray, session } from 'electron';
import path from 'node:path';
import started from 'electron-squirrel-startup';
import * as http from 'http';
import * as https from 'https';
import { URL } from 'url';

// 确保只有一个实例在运行
const gotTheLock = app.requestSingleInstanceLock();

if (!gotTheLock) {
  app.exit(0); // 使用 exit 而不是 quit，确保立即退出
} else {
  app.on('second-instance', (event, commandLine, workingDirectory) => {
    // 当运行第二个实例时，我们应该聚焦到主窗口
    const win = BrowserWindow.getAllWindows()[0];
    if (win) {
      win.show(); // 确保窗口显示
      if (win.isMinimized()) win.restore();
      win.focus();
    }
  });
}

// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (started) {
  app.exit(0); // 使用 exit 而不是 quit，确保立即退出
}

ipcMain.handle('check-url-available', async (event, rawUrl: string) => {
  try {
    const url = new URL(rawUrl);
    const lib = url.protocol === 'https:' ? https : http;

    return await new Promise((resolve) => {
      const req = lib.request(
        {
          method: 'HEAD',
          hostname: url.hostname,
          port: url.port || undefined,
          path: url.pathname + url.search,
          timeout: 3000,
        },
        (res) => {
          resolve({ ok: true, status: res.statusCode })
          req.destroy()
        }
      );

      req.on('error', (err) => {
        resolve({ ok: false, error: err.message })
      });

      req.on('timeout', () => {
        req.destroy()
        resolve({ ok: false, error: 'Timeout' })
      });

      req.end();
    })
  } catch (e) {
    return { ok: false, error: 'Invalid URL' };
  }
});

// 设置webview的cookie
ipcMain.handle('set-webview-cookie', async (event, url: string, cookie: string) => {
  try {
    const parsedUrl = new URL(url);
    const cookies: Array<{ Key: string, Value: string }> = JSON.parse(cookie);

    for (const cookieItem of cookies) {
      await session.defaultSession.cookies.set({
        url: url,
        name: cookieItem.Key.trim(),
        value: cookieItem.Value.trim(),
        domain: parsedUrl.hostname,
        path: '/',
        secure: parsedUrl.protocol === 'https:',
        httpOnly: true
      });
    }
    return true;
  } catch (error) {
    console.error('设置cookie失败:', error);
    return false;
  }
});

const createWindow = () => {
  // Create the browser window.
  const win = new BrowserWindow({
    minWidth: 800,
    minHeight: 600,
    autoHideMenuBar: true,
    center: true,
    //resizable: false,
    //minimizable: false,
    //maximizable: false,
    //alwaysOnTop: true,
    webPreferences: {
      contextIsolation: true, // 启用上下文隔离。
      nodeIntegration: false, // 禁用 Node.js 集成，提高安全性。
      preload: path.join(__dirname, 'preload.js'), // 预加载脚本
      webviewTag: true, // 启用webview标签
      //webSecurity: false, // 禁用web安全策略，允许跨域请求
      //allowRunningInsecureContent: true, // 允许运行不安全的内容
      //sandbox: false // 禁用沙箱
    },
  });

  // 隐藏顶部菜单栏
  win.setMenuBarVisibility(false);
  win.setAutoHideMenuBar(true);
  win.setMenu(null);

  // // 配置 webview 的权限
  // win.webContents.session.webRequest.onBeforeSendHeaders((details, callback) => {
  //   callback({
  //     requestHeaders: {
  //       ...details.requestHeaders,
  //       'Origin': 'http://1.12.73.211:8848'
  //     }
  //   });
  // });

  // // 配置 webview 的响应头
  // win.webContents.session.webRequest.onHeadersReceived((details, callback) => {
  //   callback({
  //     responseHeaders: {
  //       ...details.responseHeaders,
  //       'Access-Control-Allow-Origin': ['*'],
  //       'Access-Control-Allow-Methods': ['GET, POST, PUT, DELETE, OPTIONS'],
  //       'Access-Control-Allow-Headers': ['Content-Type, Authorization']
  //     }
  //   });
  // });

  // 设置session
  win.webContents.session.setPermissionRequestHandler((webContents, permission, callback) => {
    callback(true);
  });

  // 最小化到系统托盘
  win.on('minimize', () => {
    win.hide();
  });

  // 监听窗口关闭（用户点击右上角X）
  win.on('close', (event) => {
    event.preventDefault();
    win.hide();
  });

  // production a bad jabe
  const handleDevTools = () => {
    if (win.webContents.isDevToolsOpened()) {
      win.webContents.closeDevTools();
    } else {
      win.webContents.openDevTools();
    }
  };
  globalShortcut.register("CommandOrControl+Shift+I", handleDevTools);

  // and load the login.html of the app.
  if (MAIN_WINDOW_VITE_DEV_SERVER_URL) {
    win.loadURL(`${MAIN_WINDOW_VITE_DEV_SERVER_URL}/login.html`);
  } else {
    win.loadFile(path.join(__dirname, `../renderer/${MAIN_WINDOW_VITE_NAME}/login.html`));
  }

  // 创建托盘图标
  const contextMenu = Menu.buildFromTemplate([
    { label: '显示窗口', click: () => win.show() },
    { label: '退出程序', click: () => app.exit() }
  ]);
  const iconPath = path.join(process.resourcesPath, 'assets', 'tray.png');
  const tray = new Tray(iconPath);
  tray.setToolTip('中国电信-工作台');
  tray.setContextMenu(contextMenu);

  // 点击托盘图标也可以恢复窗口（可选）
  tray.on('click', () => {
    win.show();
  });
};

// 在非 macOS 平台上，关闭所有窗口时退出应用
// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.whenReady().then(() => {
  createWindow();

  // 在 macOS 平台上，点击 dock 图标时重新创建窗口
  app.on('activate', () => {
    // On OS X it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (BrowserWindow.getAllWindows().length === 0) {
      createWindow();
    }
  });
});
